using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace AdvancedToolStrip
{
    public partial class TearOffToolStrip : ToolStrip
    {
        DockingHostToolStripPanel topToolStripPanel;
        DockingHostToolStripPanel bottomToolStripPanel;
        DockingHostToolStripPanel leftToolStripPanel;
        DockingHostToolStripPanel rightToolStripPanel;

        DockingHostToolStripPanel lastDockedPanel;

        bool disableMouseCapture;
        bool isFloating;

        [Browsable(false)]
        public bool IsFloating {
            get {
                return this.isFloating;
            }
        }

        [Browsable(false)]
        public DockingHostToolStripPanel LastDockedPanel {
            get {
                return this.lastDockedPanel;
            }
        }

        [Browsable(true)]
        [Description("Determines whether the control is visible or hidden.")]
        public bool Visible {
            get
            {
                return base.Visible;
            }
            set
            {
                if (!this.isFloating)
                    base.Visible = value;
                else
                {
                    FloatingForm form = (FloatingForm)this.Parent;
                    if (form != null)
                    {
                        if (value == true) form.Show();
                        else if (value == false) form.Hide();
                    }
                }
            }
        }

        [Browsable(true)]
        [Description("Reference to the DockingHostToolStripPanel on left of the main form")]
        public DockingHostToolStripPanel TopToolStripPanel {
            get {
                return this.topToolStripPanel;
            }
            set {
                this.topToolStripPanel = value;
            }
        }

        [Browsable(true)]
        [Description("Reference to the DockingHostToolStripPanel on bottom of the main form")]
        public DockingHostToolStripPanel BottomToolStripPanel
        {
            get
            {
                return this.bottomToolStripPanel;
            }
            set
            {
                this.bottomToolStripPanel = value;
            }
        }

        [Browsable(true)]
        [Description("Reference to the DockingHostToolStripPanel on left of the main form")]
        public DockingHostToolStripPanel LeftToolStripPanel
        {
            get
            {
                return this.leftToolStripPanel;
            }
            set
            {
                this.leftToolStripPanel = value;
            }
        }

        [Browsable(true)]
        [Description("Reference to the DockingHostToolStripPanel on right of the main form")]
        public DockingHostToolStripPanel RightToolStripPanel
        {
            get
            {
                return this.rightToolStripPanel;
            }
            set
            {
                this.rightToolStripPanel = value;
            }
        }

        public TearOffToolStrip()
        {
            this.topToolStripPanel = null;
            this.bottomToolStripPanel = null;
            this.leftToolStripPanel = null;
            this.rightToolStripPanel = null;

            this.isFloating = false;
            this.disableMouseCapture = false;

            this.lastDockedPanel = null;

            InitializeComponent();
        }

        protected override void OnVisibleChanged(EventArgs e)
        {
            base.OnVisibleChanged(e);            
        }

        protected override void OnMouseDoubleClick(MouseEventArgs e)
        {
            base.OnMouseDoubleClick(e);

            if (!this.isFloating)
            {
                this.MakeFloating();
            }
        }

        protected override void OnMouseCaptureChanged(EventArgs e)
        {
            base.OnMouseCaptureChanged(e);

            if (this.disableMouseCapture == true)
            {
                this.disableMouseCapture = false;
                return;
            }

            if (this.DesignMode == true)
                return;

            if (topToolStripPanel == null
                && bottomToolStripPanel == null
                && leftToolStripPanel == null
                && rightToolStripPanel == null)
                return;

            if (!topToolStripPanel.IsPointWithinMe(MousePosition)
                && !rightToolStripPanel.IsPointWithinMe(MousePosition)
                && !bottomToolStripPanel.IsPointWithinMe(MousePosition)
                && !leftToolStripPanel.IsPointWithinMe(MousePosition))
            {
                this.MakeFloating();
            }
            else if (
                this.topToolStripPanel.IsPointWithinMe(MousePosition)
                || this.bottomToolStripPanel.IsPointWithinMe(MousePosition)
                || this.leftToolStripPanel.IsPointWithinMe(MousePosition)
                || this.rightToolStripPanel.IsPointWithinMe(MousePosition)
                )
            {
                if (this.topToolStripPanel.IsPointWithinMe(MousePosition))
                    this.DockTo(this.topToolStripPanel);
                else if (this.bottomToolStripPanel.IsPointWithinMe(MousePosition))
                    this.DockTo(this.bottomToolStripPanel);
                else if (this.leftToolStripPanel.IsPointWithinMe(MousePosition))
                    this.DockTo(this.leftToolStripPanel);
                else if (this.rightToolStripPanel.IsPointWithinMe(MousePosition))
                    this.DockTo(this.rightToolStripPanel);
            }
        }

        public void MakeFloating()
        {
            DockingHostToolStripPanel parentPanel =
                this.Parent as DockingHostToolStripPanel;

            if (parentPanel == null)
                return;

            this.disableMouseCapture = true;

            // set the flag which says we are floating
            this.isFloating = true;

            Orientation currentOrientation = this.Orientation;

            FloatingForm floatingForm = new FloatingForm();
            floatingForm.Text = this.Text;

            floatingForm.Bounds = new Rectangle(MousePosition, this.Size);
            floatingForm.Location = new Point(MousePosition.X + 25, MousePosition.Y + 25);

            // ok, remove ourselves from our parent panel and dock to the form
            this.lastDockedPanel = parentPanel;
            parentPanel.Controls.Remove(this);
            floatingForm.Controls.Add(this);

            // change our properties
            this.Stretch = true;
            this.Dock = DockStyle.Fill;
            this.GripStyle = ToolStripGripStyle.Hidden;

            // and set up events
            floatingForm.MouseCaptureChanged += new EventHandler(FloatingForm_Dropped);
            floatingForm.DockingRequested += new EventHandler(FloatingForm_DockingRequested);

            // finally show the floating toolbar
            floatingForm.Show(parentPanel.Parent);
        }

        void FloatingForm_DockingRequested(object sender, EventArgs e)
        {
            // always dock a double-clicked toolbar to the panel
            // from whence it came, or else the top tool strip panel by default
            if (this.lastDockedPanel == null)
                this.lastDockedPanel = this.topToolStripPanel;

            this.DockTo(this.lastDockedPanel);
        }

        void FloatingForm_Dropped(object sender, EventArgs e)
        {
            this.OnMouseCaptureChanged(e);
        }

        public void DockTo(DockingHostToolStripPanel panel)
        {
            if (!this.IsFloating)
                return;

            // if NULL is specified for the panel to which to dock
            // of if our Parent is a ToolStripPanel (i.e., we're already docked)
            // stop and return to the caller
            if ((panel == null)
                || (this.Parent is ToolStripPanel))
                return;

            this.isFloating = false;
            FloatingForm floatingForm = (FloatingForm)this.Parent;

            floatingForm.Hide();

            this.Stretch = false; this.GripStyle = ToolStripGripStyle.Hidden;
            floatingForm.Controls.Remove(this);
            panel.Controls.Add(this);

            //this.LayoutStyle = ToolStripLayoutStyle.StackWithOverflow;
            //this.AutoSize = true;

            floatingForm.Close();

        }

        private void TearOffToolStrip_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            // this is done to prevent the action of the user clicking a button triggering a tearoff
            this.disableMouseCapture = true;
        }
    }
}
